ButtonWT = Core.class(Sprite)
local textColorUp = 0x000000;
local textColorDown = 0x000000;
local backgroundColorUp = {r = 1, g = 1, b = 1, a = 1}
local backgroundColorDown = {r = 0.8, g = 0.8, b = 0.8, a = 0.8}
function ButtonWT:init(color, value, font, width, height)
	local bitmap
	if color == nil then
		bitmap = Bitmap.new(Texture.new("images/button-up.png", true))
	else
		bitmap = Bitmap.new(Texture.new("images/button-up.png", true))
	end
	
	if width then
		bitmapWidth = bitmap:getWidth()
		bitmap:setScaleX(width / bitmapWidth)
	end	
	if height then
		bitmapHeight = bitmap:getHeight()
		bitmap:setScaleY(height / bitmapHeight)
	end		
	self.bitmap = bitmap
	self:addChild(self.bitmap)

	if value then
		self.value = TextField.new(font, value);
		self.value:setTextColor( textColorUp )
		self.value:setAnchorPoint(0.5, 0.5);
		self.value:setPosition(bitmap:getWidth()/2, bitmap:getHeight()/2);
	end
	self.focus = false
	

	if self.value and (not self:contains(self.value)) then
		self:addChild(self.value)
	end
 
	-- register to all mouse and touch events
	self:addEventListener(Event.MOUSE_DOWN, self.onMouseDown, self)
	self:addEventListener(Event.MOUSE_MOVE, self.onMouseMove, self)
	self:addEventListener(Event.MOUSE_UP, self.onMouseUp, self)
 
	self:addEventListener(Event.TOUCHES_BEGIN, self.onTouchesBegin, self)
	self:addEventListener(Event.TOUCHES_MOVE, self.onTouchesMove, self)
	self:addEventListener(Event.TOUCHES_END, self.onTouchesEnd, self)
	self:addEventListener(Event.TOUCHES_CANCEL, self.onTouchesCancel, self)
end
 
function ButtonWT:onMouseDown(event)
	if self:hitTestPoint(event.x, event.y) then
		self.focus = true	
		self:updateVisualState()
		event:stopPropagation()
	end
end
 
function ButtonWT:onMouseMove(event)
	if self.focus then
		if not self:hitTestPoint(event.x, event.y) then	
			self.focus = false
			self:updateVisualState()
		end
		event:stopPropagation()
	end
end

function ButtonWT:onMouseUp(event)
	if self.focus then
		sounds:play("buttonClick")		
		self.focus = false
		self:updateVisualState()
		self:dispatchEvent(Event.new("click"))	-- button is clicked, dispatch "click" event
		event:stopPropagation()
	end
end
 
function ButtonWT:onTouchesBegin(event)
	if self.focus then
		event:stopPropagation()
	end
end
 
function ButtonWT:onTouchesMove(event)
	if self.focus then
		event:stopPropagation()
	end
end
 
function ButtonWT:onTouchesEnd(event)
	if self.focus then
		event:stopPropagation()
	end
end
 
function ButtonWT:onTouchesCancel(event)
	if self.focus then
		self.focus = false;
		self:updateVisualState()
		event:stopPropagation()
	end
end

function ButtonWT:updateVisualState()
	if self.focus then
		self.bitmap:setColorTransform(backgroundColorDown.r, backgroundColorDown.g, backgroundColorDown.b, backgroundColorDown.a)
		if self.value and self:contains(self.value) then
			self.value:setTextColor(textColorDown)
		end
	else
		self.bitmap:setColorTransform(backgroundColorUp.r, backgroundColorUp.g, backgroundColorUp.b, backgroundColorUp.a)
		if self.value and self:contains(self.value) then
			self.value:setTextColor(textColorUp)
		end
	end
end

function ButtonWT:updateText(newText)
	if self.value then
		self.value:setText(newText)
	end
end