--[[
*************************************************************
 * This script is developed by Arturs Sosins aka ar2rsawseen, http://appcodingeasy.com
 * Feel free to distribute and modify code, but keep reference to its creator
 *
 * Gideros Game Template for developing games. Includes: 
 * Start scene, pack select, level select, settings, score system and much more
 *
 * For more information, examples and online documentation visit: 
 * http://appcodingeasy.com/Gideros-Mobile/Gideros-Mobile-Game-Template
**************************************************************
]]--
local function randomize(elem)
	elem:setPosition(math.random(-1000, 1000), math.random(-1000, 1000))
	elem:setRotation(math.random(0, 360))
	elem:setAlpha(math.random())
	elem:setScale(math.random())
end

options = gideros.class(Sprite)

function options:init()
	local buttonsWidth = 180
	lib.fillBackground('images/background/www.png', self)
	--here we'd probably want to set up a background picture
	local screen = Bitmap.new(Texture.new("images/gideros_mobile.png", conf.textureFilter))
	self:addChild(screen)
	screen:setPosition((conf.width-screen:getWidth())/2, (conf.height-screen:getHeight())/2)
	
	--create layer for menu buttons
	--local layer = Popup.new({easing = conf.easing})
	--layer:setFillStyle(Shape.SOLID, 0xff0000, 0.2)
	--layer:setLineStyle(5, 0x000000)
	--layer:setPosition(conf.width/2, conf.height/2)
	--layer:setScale(0)
	
	--self:addChild(layer)
	
	local menu = VerticalView.new({padding = 20, easing = conf.easing})
	self:addChild(menu)
	--layer:addChild(menu)

	local musicOnButton = ButtonWT.new("blue", 'Music on', conf.buttonFont, buttonsWidth)
	local musicOffButton = ButtonWT.new("blue", 'Music off', conf.buttonFont, buttonsWidth)
	
	musicOnButton:addEventListener("click", function()
			menu:replaceChild(musicOnButton, musicOffButton)
			music:off()
			sets:set("music", false, true)
	end)
	
	local musicIndex = 1
	musicOffButton:addEventListener("click", function()
			menu:replaceChild(musicOffButton, musicOnButton)
			music = Music.new(conf.musicList[musicIndex])
			music:on()
			sets:set("music", true, true)
			musicIndex = musicIndex + 1
			if musicIndex > #conf.musicList then
				musicIndex = 1
			end
	end)
	
	if sets:get("music") then
		randomize(musicOnButton)
		menu:addChild(musicOnButton)
	else
		randomize(musicOffButton)
		menu:addChild(musicOffButton)
	end
	
	local soundsOnButton = ButtonWT.new("pink", 'Sounds on', conf.buttonFont, buttonsWidth)
	local soundsOffButton = ButtonWT.new("pink", 'Sounds off', conf.buttonFont, buttonsWidth)
	
	soundsOnButton:addEventListener("click", 
		function()
			menu:replaceChild(soundsOnButton, soundsOffButton)
			sounds:off()
		end
	)
	
	soundsOffButton:addEventListener("click", 
		function()
			menu:replaceChild(soundsOffButton, soundsOnButton)
			sounds:on()
		end
	)
	
	if sets:get("sounds") then
		randomize(soundsOnButton)
		menu:addChild(soundsOnButton)
	else
		randomize(soundsOffButton)
		menu:addChild(soundsOffButton)
	end
	
	local backButton = ButtonWT.new("green", 'Back', conf.buttonFont, buttonsWidth)
	backButton:addEventListener("click", 
		function()
		menu:reverse()
		sceneManager:changeScene("start", 1, conf.transition, conf.easing) 
	end
	)
	randomize(backButton)
	menu:addChild(backButton)
	
	--position menu in center
	menu:setAnchorPoint(0.5, 0.5)	
	menu:setPosition((conf.width)/2, (conf.height)/2)
	--menu:setPosition(-menu:getWidth()/2, -menu:getHeight()/2)
	
	--layer:show()
	
end